<?php
/*
Plugin Name: DEV Backup Plugin (EasyToolsWP)
Description: A plugin to create backups of WordPress files and database and download them.
Version: 1.4
Author: EasyToolsWP.com
Author URI: https://www.easytoolswp.com
*/

// Ensure WordPress is loaded
defined('ABSPATH') or die('No direct access allowed!');

// Add menu item
add_action('admin_menu', 'etwp_add_admin_menu');

function etwp_add_admin_menu() {
    add_submenu_page(
        'tools.php',
        'ETWP Backup',
        'ETWP Backup',
        'manage_options',
        'etwp-backup',
        'etwp_backup_page'
    );
}

// Add content to the backup page
function etwp_backup_page() {
    ?>
    <div class="wrap">
        <h1>Backup Tools</h1>

        <!-- File Backup Form -->
        <h2>File Backup</h2>
        <form id="etwp-file-backup-form">
            <select name="etwp_file_backup_option">
                <option value="all">WordPress dir</option>
                <option value="themes">Themes only</option>
                <option value="plugins">Plugins only</option>
                <option value="uploads">Uploads only</option>
            </select>
            <input type="button" id="etwp-file-backup-button" value="Create File Backup" class="button button-primary"/>
        </form>
        
        <div id="etwp-file-popup" class="etwp-popup" style="display:none;">
            <div class="etwp-popup-content">
                <h2>Creating zip file, this can take a while.... please wait</h2>
                <a href="#" id="etwp-file-stop-backup" class="etwp-stop-backup" onclick="etwp_file_stop_backup();">Stop Backup</a>
            </div>
        </div>

        <h2>Database Backup</h2>
        <form id="etwp-db-backup-form">
            <input type="button" id="etwp-db-backup-button" value="Create Database Backup" class="button button-primary"/>
        </form>
        
        <div id="etwp-db-popup" class="etwp-popup" style="display:none;">
            <div class="etwp-popup-content">
                <h2>Creating SQL file, this can take a while.... please wait</h2>
                <a href="#" id="etwp-db-stop-backup" class="etwp-stop-backup" onclick="etwp_db_stop_backup();">Stop Backup</a>
            </div>
        </div>
        
        <style>
            .etwp-popup {
                position: fixed;
                top: 0;
                left: 0;
                width: 100%;
                height: 100%;
                background: rgba(0,0,0,0.5);
                display: flex;
                align-items: center;
                justify-content: center;
                z-index: 1000;
            }
            .etwp-popup-content {
                background: white;
                padding: 20px;
                border-radius: 5px;
                text-align: center;
            }
            .etwp-stop-backup {
                display: block;
                margin-top: 20px;
                color: red;
                text-decoration: underline;
                cursor: pointer;
            }
        </style>
        
        <script>
            // File Backup
            document.getElementById('etwp-file-backup-button').onclick = function() {
                var option = document.querySelector('select[name="etwp_file_backup_option"]').value;
                
                // Show file backup popup
                document.getElementById('etwp-file-popup').style.display = 'flex';
                
                var xhr = new XMLHttpRequest();
                xhr.open('POST', ajaxurl, true);
                xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                xhr.onload = function() {
                    if (xhr.status === 200) {
                        var response = JSON.parse(xhr.responseText);
                        if (response.success) {
                            // Hide popup and redirect to download link
                            document.getElementById('etwp-file-popup').style.display = 'none';
                            window.location.href = response.data.download_url;
                        } else {
                            alert('An error occurred: ' + response.data.message);
                            document.getElementById('etwp-file-popup').style.display = 'none';
                        }
                    } else {
                        alert('An error occurred: ' + xhr.statusText);
                        document.getElementById('etwp-file-popup').style.display = 'none';
                    }
                };
                xhr.send('action=etwp_create_file_backup&option=' + encodeURIComponent(option));
            };

            function etwp_file_stop_backup() {
                // Redirect to the same page to refresh and stop backup
                window.location.href = window.location.href;
            }

            // Database Backup
            document.getElementById('etwp-db-backup-button').onclick = function() {
                // Show database backup popup
                document.getElementById('etwp-db-popup').style.display = 'flex';
                
                var xhr = new XMLHttpRequest();
                xhr.open('POST', ajaxurl, true);
                xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                xhr.onload = function() {
                    if (xhr.status === 200) {
                        var response = JSON.parse(xhr.responseText);
                        if (response.success) {
                            // Hide popup and redirect to download link
                            document.getElementById('etwp-db-popup').style.display = 'none';
                            window.location.href = response.data.download_url;
                        } else {
                            alert('An error occurred: ' + response.data.message);
                            document.getElementById('etwp-db-popup').style.display = 'none';
                        }
                    } else {
                        alert('An error occurred: ' + xhr.statusText);
                        document.getElementById('etwp-db-popup').style.display = 'none';
                    }
                };
                xhr.send('action=etwp_create_db_backup');
            };

            function etwp_db_stop_backup() {
                // Redirect to the same page to refresh and stop backup
                window.location.href = window.location.href;
            }
        </script>
    </div>
    <?php
}

// AJAX handling for creating file backup
add_action('wp_ajax_etwp_create_file_backup', 'etwp_create_file_backup');

function etwp_create_file_backup() {
    global $wpdb; // Ensure $wpdb is available

    $option = isset($_POST['option']) ? sanitize_text_field($_POST['option']) : 'all';
    $upload_dir = wp_upload_dir();
    $root_dir = ABSPATH;
    $zip_name = 'wordpress-backup-' . date('Y-m-d-H-i-s') . '.zip';
    $zip_file = $upload_dir['path'] . '/' . $zip_name;

    $zip = new ZipArchive();
    if ($zip->open($zip_file, ZipArchive::CREATE | ZipArchive::OVERWRITE) !== TRUE) {
        wp_send_json_error(array('message' => 'Unable to open zip file for writing'));
    }

    // Select the desired directory
    switch ($option) {
        case 'themes':
            $dir = $root_dir . 'wp-content/themes/';
            break;
        case 'plugins':
            $dir = $root_dir . 'wp-content/plugins/';
            break;
        case 'uploads':
            $dir = $root_dir . 'wp-content/uploads/';
            break;
        default:
            $dir = $root_dir;
            break;
    }

    etwp_add_files_to_zip($zip, $dir, '');

    $zip->close();

    if (file_exists($zip_file)) {
        wp_send_json_success(array('download_url' => $upload_dir['url'] . '/' . $zip_name));
    } else {
        wp_send_json_error(array('message' => 'The zip file was not created'));
    }
}

// Check if EasyTools-v2 plugin is active
function etwp_backup_check_easytools_plugin() {
    include_once(ABSPATH . 'wp-admin/includes/plugin.php');

    if (!is_plugin_active('easytoolswp-v2/easytoolswp-v2.php')) {
        add_action('admin_notices', 'etwp_backup_easytools_missing_notice');
    }
}

// Display notice if EasyTools-v2 plugin is missing
function etwp_backup_easytools_missing_notice() {
    ?>
    <div class="notice notice-warning is-dismissible">
        <p><?php _e('Get more add-ons with <strong>EasyToolsWP V2</strong> - Download <a href="https://www.easytoolswp.com" target="_blank">EasyToolsWP V2</a>.', 'your-text-domain'); ?></p>
    </div>
    <?php
}

// Hook into admin initialization to check for EasyTools-v2 plugin
add_action('admin_init', 'etwp_backup_check_easytools_plugin');

// Add files to zip
function etwp_add_files_to_zip($zip, $dir, $base = '') {
    $files = glob($dir . '*');
    foreach ($files as $file) {
        if (is_dir($file)) {
            etwp_add_files_to_zip($zip, $file . '/', $base . basename($file) . '/');
        } else {
            $relative_path = $base . basename($file);
            $zip->addFile($file, $relative_path);
        }
    }
}

// AJAX handling for creating database backup
add_action('wp_ajax_etwp_create_db_backup', 'etwp_create_db_backup');

function etwp_create_db_backup() {
    global $wpdb; // Ensure $wpdb is available

    // Ensure $wpdb is available
    if ( ! isset($wpdb) || ! $wpdb instanceof wpdb ) {
        wp_send_json_error(array('message' => 'Database connection is not available.'));
        return;
    }
    
    // Define the backup file name
    $backup_name = 'wordpress-database-backup-' . date('Y-m-d-H-i-s') . '.sql';
    $backup_file = WP_CONTENT_DIR . '/uploads/' . $backup_name;

    // Create a backup file using the wpdb object
    $tables = $wpdb->get_results('SHOW TABLES', ARRAY_N);
    $output = '';

    foreach ($tables as $table) {
        $table_name = $table[0];
        $output .= "DROP TABLE IF EXISTS `$table_name`;\n";
        $create_table = $wpdb->get_row("SHOW CREATE TABLE `$table_name`", ARRAY_N);
        $output .= $create_table[1] . ";\n\n";
        
        $rows = $wpdb->get_results("SELECT * FROM `$table_name`", ARRAY_A);
        foreach ($rows as $row) {
            $values = array_map(function($value) use ($wpdb) {
                return is_null($value) ? 'NULL' : $wpdb->prepare('%s', $value);
            }, array_values($row));
            $output .= "INSERT INTO `$table_name` (" . implode(', ', array_keys($row)) . ") VALUES (" . implode(', ', $values) . ");\n";
        }
        $output .= "\n\n";
    }

    // Write the output to file
    file_put_contents($backup_file, $output);

    if (file_exists($backup_file)) {
        wp_send_json_success(array('download_url' => content_url('uploads/' . $backup_name)));
    } else {
        wp_send_json_error(array('message' => 'The SQL file was not created'));
    }
}
?>
